

   MODULE module_check_a_mundo
      USE module_state_description
      USE module_wrf_error
      USE module_configure
      IMPLICIT NONE
   CONTAINS
   SUBROUTINE check_nml_consistency
      IMPLICIT NONE
      INTEGER :: i
      DO i = 2, model_config_rec % max_dom
         IF ( model_config_rec % sf_surface_physics(i) .NE. &
              model_config_rec % sf_surface_physics(i-1) ) THEN
            wrf_err_message = '--- ERROR: sf_surface_physics must be equal for all domains '
            CALL wrf_message ( wrf_err_message )
            wrf_err_message = '--- Fix sf_surface_physics in namelist.input '
            CALL wrf_error_fatal3("<stdin>",56,&
TRIM( wrf_err_message ) )
         END IF
      ENDDO
      IF ( ( model_config_rec%fractional_seaice .EQ. 0 ).AND. &
              ( model_config_rec%tice2tsk_if2cold ) ) THEN
            wrf_err_message = '--- WARNING: You set tice2tsk_if2cold = .true.,  but fractional_seaice = 0'
            CALL wrf_message ( wrf_err_message )
            wrf_err_message = '--- WARNING: tice2tsk_if2cold will have no effect on results.'
            CALL wrf_message ( wrf_err_message )
      END IF
      DO i = 1, model_config_rec % max_dom
         IF ( ( model_config_rec%fine_input_stream(i) .NE. 0 ).AND. &
              ( model_config_rec%io_form_auxinput2 .EQ. 0 ) ) THEN
            wrf_err_message = '--- ERROR: If fine_input_stream /= 0, io_form_auxinput2 must be /= 0'
            CALL wrf_message ( wrf_err_message )
            wrf_err_message = '--- Set io_form_auxinput2 in the time_control namelist (probably to 2).'
            CALL wrf_error_fatal3("<stdin>",83,&
TRIM( wrf_err_message ) )
         END IF
      ENDDO
   model_config_rec % stoch_force_global_opt=0
   DO i = 1, model_config_rec % max_dom
         IF ( model_config_rec % stoch_force_opt(i) .EQ. 1) then
           model_config_rec % stoch_force_global_opt=1
         endif
   ENDDO
      IF ( model_config_rec%sst_update .EQ. 0 ) THEN
         model_config_rec%io_form_auxinput4 = 0
         DO i = 1, model_config_rec % max_dom
            WRITE (wrf_err_message, FMT='(A,A)') '--- NOTE: sst_update is 0, ', &
                  'setting io_form_auxinput4 = 0 and auxinput4_interval = 0 for all domains'
            CALL wrf_message ( wrf_err_message )
            model_config_rec%auxinput4_interval(i) = 0
            model_config_rec%auxinput4_interval_y(i) = 0
            model_config_rec%auxinput4_interval_d(i) = 0
            model_config_rec%auxinput4_interval_h(i) = 0
            model_config_rec%auxinput4_interval_m(i) = 0
            model_config_rec%auxinput4_interval_s(i) = 0
         ENDDO
      ELSE
         IF ( model_config_rec%io_form_auxinput4 .EQ. 0 ) THEN
            wrf_err_message = '--- ERROR: If sst_update /= 0, io_form_auxinput4 must be /= 0'
            CALL wrf_message ( wrf_err_message )
            wrf_err_message = '--- Set io_form_auxinput4 in the time_control namelist (probably to 2).'
            CALL wrf_error_fatal3("<stdin>",126,&
TRIM( wrf_err_message ) )
         END IF
      END IF
      DO i = 1, model_config_rec % max_dom
         IF ( ( model_config_rec%grid_sfdda(i) .EQ. 1 ).AND. &
              ( model_config_rec%grid_fdda (i) .NE. 1 ) ) THEN
            wrf_err_message = '--- ERROR: If grid_sfdda = 1, then grid_fdda must also = 1 for that domain '
            CALL wrf_message ( wrf_err_message )
            wrf_err_message = '--- Change grid_fdda or grid_sfdda in namelist.input '
            CALL wrf_error_fatal3("<stdin>",142,&
TRIM( wrf_err_message ) )
         END IF
      ENDDO
      DO i = 1, model_config_rec % max_dom
         IF ( model_config_rec%grid_fdda(i) .EQ. 0 ) THEN
            WRITE (wrf_err_message, FMT='(A,I6,A)') '--- NOTE: grid_fdda is 0 for domain ', &
                         i, ', setting gfdda interval and ending time to 0 for that domain.'
            CALL wrf_message ( wrf_err_message )
            model_config_rec%gfdda_end_y(i) = 0
            model_config_rec%gfdda_end_d(i) = 0
            model_config_rec%gfdda_end_h(i) = 0
            model_config_rec%gfdda_end_m(i) = 0
            model_config_rec%gfdda_end_s(i) = 0
            model_config_rec%gfdda_interval(i) = 0
            model_config_rec%gfdda_interval_y(i) = 0
            model_config_rec%gfdda_interval_d(i) = 0
            model_config_rec%gfdda_interval_h(i) = 0
            model_config_rec%gfdda_interval_m(i) = 0
            model_config_rec%gfdda_interval_s(i) = 0
         END IF
         IF ( ( model_config_rec%grid_sfdda(i) .EQ. 0 ) .AND. &
              ( model_config_rec%pxlsm_soil_nudge(i) .EQ. 0 ) ) THEN
            WRITE (wrf_err_message, FMT='(A,I6,A)') &
                         '--- NOTE: both grid_sfdda and pxlsm_soil_nudge are 0 for domain ', &
                         i, ', setting sgfdda interval and ending time to 0 for that domain.'
            CALL wrf_message ( wrf_err_message )
            model_config_rec%sgfdda_end_y(i) = 0
            model_config_rec%sgfdda_end_d(i) = 0
            model_config_rec%sgfdda_end_h(i) = 0
            model_config_rec%sgfdda_end_m(i) = 0
            model_config_rec%sgfdda_end_s(i) = 0
            model_config_rec%sgfdda_interval(i) = 0
            model_config_rec%sgfdda_interval_y(i) = 0
            model_config_rec%sgfdda_interval_d(i) = 0
            model_config_rec%sgfdda_interval_h(i) = 0
            model_config_rec%sgfdda_interval_m(i) = 0
            model_config_rec%sgfdda_interval_s(i) = 0
         END IF
         IF ( model_config_rec%obs_nudge_opt(i) .EQ. 0 ) THEN
            WRITE (wrf_err_message, FMT='(A,I6,A)') '--- NOTE: obs_nudge_opt is 0 for domain ', &
                         i, ', setting obs nudging interval and ending time to 0 for that domain.'
            CALL wrf_message ( wrf_err_message )
            model_config_rec%fdda_end(i) = 0
            model_config_rec%auxinput11_interval(i) = 0
            model_config_rec%auxinput11_interval_y(i) = 0
            model_config_rec%auxinput11_interval_d(i) = 0
            model_config_rec%auxinput11_interval_h(i) = 0
            model_config_rec%auxinput11_interval_m(i) = 0
            model_config_rec%auxinput11_interval_s(i) = 0
            model_config_rec%auxinput11_end(i) = 0
            model_config_rec%auxinput11_end_y(i) = 0
            model_config_rec%auxinput11_end_d(i) = 0
            model_config_rec%auxinput11_end_h(i) = 0
            model_config_rec%auxinput11_end_m(i) = 0
            model_config_rec%auxinput11_end_s(i) = 0
         END IF
      ENDDO
      IF ( MAXVAL( model_config_rec%grid_fdda ) .EQ. 0 ) THEN
         model_config_rec%io_form_gfdda = 0
      ELSE
         IF ( model_config_rec%io_form_gfdda .EQ. 0 ) THEN
            wrf_err_message = '--- ERROR: If grid_fdda /= 0, io_form_gfdda must be /= 0'
            CALL wrf_message ( wrf_err_message )
            wrf_err_message = '--- Set io_form_gfdda in the time_control namelist (probably to 2).'
            CALL wrf_error_fatal3("<stdin>",229,&
TRIM( wrf_err_message ) )
         END IF
      END IF
      IF ( MAXVAL( model_config_rec%grid_sfdda ) .EQ. 0 ) THEN
         model_config_rec%io_form_sgfdda = 0
      ELSE
         IF ( model_config_rec%io_form_sgfdda .EQ. 0 ) THEN
            wrf_err_message = '--- ERROR: If grid_sfdda /= 0, io_form_sgfdda must be /= 0'
            CALL wrf_message ( wrf_err_message )
            wrf_err_message = '--- Set io_form_sgfdda in the time_control namelist (probably to 2).'
            CALL wrf_error_fatal3("<stdin>",240,&
TRIM( wrf_err_message ) )
         END IF
      END IF
      model_config_rec%auxinput10_begin_d = model_config_rec%gfdda_begin_d
      model_config_rec%auxinput10_begin_h = model_config_rec%gfdda_begin_h
      model_config_rec%auxinput10_begin_m = model_config_rec%gfdda_begin_m
      model_config_rec%auxinput10_begin_s = model_config_rec%gfdda_begin_s
      model_config_rec%auxinput10_begin_y = model_config_rec%gfdda_begin_y
      model_config_rec%auxinput10_end_d = model_config_rec%gfdda_end_d
      model_config_rec%auxinput10_end_h = model_config_rec%gfdda_end_h
      model_config_rec%auxinput10_end_m = model_config_rec%gfdda_end_m
      model_config_rec%auxinput10_end_s = model_config_rec%gfdda_end_s
      model_config_rec%auxinput10_end_y = model_config_rec%gfdda_end_y
      model_config_rec%auxinput10_inname = model_config_rec%gfdda_inname
      model_config_rec%auxinput10_interval = model_config_rec%gfdda_interval
      model_config_rec%auxinput10_interval_d = model_config_rec%gfdda_interval_d
      model_config_rec%auxinput10_interval_h = model_config_rec%gfdda_interval_h
      model_config_rec%auxinput10_interval_m = model_config_rec%gfdda_interval_m
      model_config_rec%auxinput10_interval_s = model_config_rec%gfdda_interval_s
      model_config_rec%auxinput10_interval_y = model_config_rec%gfdda_interval_y
      model_config_rec%io_form_auxinput10 = model_config_rec%io_form_gfdda
      model_config_rec%auxinput9_begin_d = model_config_rec%sgfdda_begin_d
      model_config_rec%auxinput9_begin_h = model_config_rec%sgfdda_begin_h
      model_config_rec%auxinput9_begin_m = model_config_rec%sgfdda_begin_m
      model_config_rec%auxinput9_begin_s = model_config_rec%sgfdda_begin_s
      model_config_rec%auxinput9_begin_y = model_config_rec%sgfdda_begin_y
      model_config_rec%auxinput9_end_d = model_config_rec%sgfdda_end_d
      model_config_rec%auxinput9_end_h = model_config_rec%sgfdda_end_h
      model_config_rec%auxinput9_end_m = model_config_rec%sgfdda_end_m
      model_config_rec%auxinput9_end_s = model_config_rec%sgfdda_end_s
      model_config_rec%auxinput9_end_y = model_config_rec%sgfdda_end_y
      model_config_rec%auxinput9_inname = model_config_rec%sgfdda_inname
      model_config_rec%auxinput9_interval = model_config_rec%sgfdda_interval
      model_config_rec%auxinput9_interval_d = model_config_rec%sgfdda_interval_d
      model_config_rec%auxinput9_interval_h = model_config_rec%sgfdda_interval_h
      model_config_rec%auxinput9_interval_m = model_config_rec%sgfdda_interval_m
      model_config_rec%auxinput9_interval_s = model_config_rec%sgfdda_interval_s
      model_config_rec%auxinput9_interval_y = model_config_rec%sgfdda_interval_y
      model_config_rec%io_form_auxinput9 = model_config_rec%io_form_sgfdda
      IF (model_config_rec%prec_acc_dt(1) .gt. 0.) model_config_rec%prec_acc_opt = 1
      IF (model_config_rec%bucket_mm .gt. 0.) model_config_rec%bucketr_opt = 1
   END SUBROUTINE
   SUBROUTINE set_physics_rconfigs
      IMPLICIT NONE
      IF (( model_config_rec % ra_lw_physics(1) .EQ. CAMLWSCHEME ) .OR. &
          ( model_config_rec % ra_sw_physics(1) .EQ. CAMSWSCHEME )) THEN
         model_config_rec % paerlev = 29
         model_config_rec % levsiz = 59
         model_config_rec % cam_abs_dim1 = 4
         model_config_rec % cam_abs_dim2 = model_config_rec % e_vert(1)
         wrf_err_message = '--- NOTE: CAM radiation is in use, setting:  ' // &
                           'paerlev=29, levsiz=59, cam_abs_dim1=4, cam_abs_dim2=e_vert'
         CALL wrf_message ( wrf_err_message )
      END IF
      IF ( model_config_rec % sf_surface_physics(1) .EQ. 0 ) &
           model_config_rec % num_soil_layers = 5
      IF ( model_config_rec % sf_surface_physics(1) .EQ. SLABSCHEME ) &
           model_config_rec % num_soil_layers = 5
      IF ( model_config_rec % sf_surface_physics(1) .EQ. LSMSCHEME ) &
           model_config_rec % num_soil_layers = 4
      IF ( model_config_rec % sf_surface_physics(1) .EQ. RUCLSMSCHEME) &
           model_config_rec % num_soil_layers = 6
      IF ( model_config_rec % sf_surface_physics(1) .EQ. PXLSMSCHEME ) &
           model_config_rec % num_soil_layers = 2
      IF ( model_config_rec % sf_surface_physics(1) .EQ. 88 ) &
           model_config_rec % num_soil_layers = 4
      WRITE (wrf_err_message, FMT='(A,I6)') '--- NOTE: num_soil_layers has been set to ', &
                                             model_config_rec % num_soil_layers
      CALL wrf_message ( wrf_err_message )
   END SUBROUTINE set_physics_rconfigs
   END MODULE module_check_a_mundo
